# -*- coding: utf-8 -*-
# authentic2_cut - Authentic2 plugin for CUT
# Copyright (C) 2017  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from django.utils.timezone import now
from django.contrib import messages
from django.contrib.auth import get_user_model
from django.views.generic.base import TemplateView

from authentic2.views import EditProfile
from authentic2.manager.views import SimpleSubTableView
from authentic2.manager.user_views import UserEditView

from .custom_settings import CORE_ATTRIBUTES
from . import tables, models


class EditCoreView(EditProfile):
    template_names = ['authentic2/cut-edit-core.html']
    fields = [
        'first_name',
        'last_name',
        'title',
        'birthdate',
        'birthplace',
        'birthcountry',
    ]

    @classmethod
    def get_fields(cls):
        fields, labels = super(EditCoreView, cls).get_fields()
        return [field for field in fields if field in cls.fields], labels

edit_core = EditCoreView.as_view()


class EditCrownView(EditProfile):
    template_names = ['authentic2/cut-edit-crown.html']
    fields = [
        'first_name',
        'last_name',
        'title',
        'birthdate',
        'birthplace',
        'birthcountry',
    ]

    @classmethod
    def get_fields(cls):
        fields, labels = super(EditCrownView, cls).get_fields()
        return [field for field in fields if field not in cls.fields], labels

edit_crown = EditCrownView.as_view()


class UserEditCoreView(UserEditView):
    template_name = 'authentic2/cut_manager_user_edit_core.html'
    permissions = ['custom_user.cut_validate_user']

    def get_title(self):
        if self.object.attributes.validated:
            return u'Modifier les attributs cœurs'
        return u'Valider les attributs cœurs'

    def get_fields(self):
        fields = super(UserEditCoreView, self).get_fields()
        return [field for field in CORE_ATTRIBUTES if field in fields]

    def get_form(self, *args, **kwargs):
        form = super(UserEditCoreView, self).get_form(*args, **kwargs)
        for field in form.fields.values():
            field.required = True
        if 'birthdate' in form.fields:
            form.fields['birthdate'].widget.options['endDate'] = '+0d'
        return form

    def form_valid(self, form):
        response = super(UserEditCoreView, self).form_valid(form)
        if not form.instance.attributes.validated:
            models.Journal.objects.create(
                actor=self.request.user,
                subject=self.object,
                message='validation du compte')
            form.instance.attributes.validated = True
            form.instance.attributes.validation_context = 'office'
            form.instance.attributes.validation_date = now().date()
            messages.info(self.request, u'Le compte a été validé.')
        else:
            models.Journal.objects.create(
                actor=self.request.user,
                subject=self.object,
                message='modification des données cœur')
            messages.info(self.request, u'Les données cœur ont été modifié.')
        return response

manager_user_edit_core = UserEditCoreView.as_view()


class UserActionsJournal(SimpleSubTableView):
    model = get_user_model()
    table_class = tables.UserActionsTable
    template_name = 'authentic2/cut_manager_user_actions_journal.html'
    permissions = ['custom_user.view_user']
    filter_table_by_perm = False

    def get_table_queryset(self):
        return self.object.actor_journal.all()

user_actions_journal = UserActionsJournal.as_view()


class UserModificationsJournal(SimpleSubTableView):
    model = get_user_model()
    table_class = tables.UserModificationsTable
    template_name = 'authentic2/cut_manager_user_modifications_journal.html'
    permissions = ['custom_user.view_user']
    filter_table_by_perm = False

    def get_table_queryset(self):
        return self.object.subject_journal.all()

user_modifications_journal = UserModificationsJournal.as_view()


class CGU(TemplateView):
    template_name = 'authentic2/cut_cgu.html'

    def get_context_data(self, **kwargs):
        kwargs['cut_base_url'] = self.request.build_absolute_uri('/')
        return super(CGU, self).get_context_data(**kwargs)

cgu = CGU.as_view()
