from django.conf import settings
import urlparse
import logging

from authentic2.utils import same_domain


class CUTMiddleware(object):
    '''Extract requesting domain from URL and set value on request and in session.

       Domains extracted from ?next=<url> of ?redirect_uri=<url> parameters are matched against the
       domains list of domain patterns.

       Configuration:

            A2_CUT_PARTNERS = [
                {
                    'domains': ['.lyon.fr'],
                    'logo_url': 'https://www.lyon.fr/static/logo_cut.png',
                    'color': '#452334',
                    'name': u'Ville de Lyon',
                }
            }

       Use in templates:

       {% if request.partner %}{{ request.partner.name }}{% endif %}
    '''
    MATCHES = {}

    def process_request(self, request):
        domain = None
        next_url = request.GET.get('next')
        if next_url or 'redirect_uri' in request.GET:
            url = next_url or request.GET.get('redirect_uri')
            netloc = urlparse.urlparse(url).netloc
            if netloc:
                domain = netloc.split(':')[-1]

        if not domain:
            domain = request.session.get('cut_domain')

        if not domain:
            request.partner = None
        else:
            if domain not in self.MATCHES:
                for partner_def in getattr(settings, 'A2_CUT_PARTNERS', []):
                    patterns = partner_def.get('domains', [])
                    for pattern in patterns:
                        if same_domain(domain, pattern):
                            break
                    else:
                        continue
                    self.MATCHES[domain] = partner_def
                    request.session['cut_domain'] = domain
                    break
                else:
                    # when adding a domain, you must reload, must thing of emptying the cache
                    # sometimes
                    self.MATCHES[domain] = None
                    request.session['cut_domain'] = None
                    request.session['cut_next'] = None

            if domain and domain in self.MATCHES:
                request.partner = self.MATCHES[domain]
                if request.session.get('cut_domain') != domain:
                    request.session['cut_domain'] = domain
                    request.session['cut_next'] = None
                if isinstance(request.partner, dict):
                    request.partner = request.partner.copy()
                    if next_url and next_url.startswith('http'):
                        request.partner['url'] = next_url
                        request.session['cut_next'] = next_url
                    elif request.session.get('cut_next'):
                        request.partner['url'] = request.session['cut_next']
            else:
                request.partner = None
            return None
